@@ -1,22 +1,27 @@
 """GreyNoise API client."""
 
+import logging
 import re
 from collections import OrderedDict
 
 import cachetools
 import more_itertools
 import requests
-import structlog
 
 from greynoise.__version__ import __version__
 from greynoise.api.analyzer import Analyzer
 from greynoise.api.filter import Filter
 from greynoise.exceptions import RateLimitError, RequestFailure
-from greynoise.util import configure_logging, load_config, validate_ip
+from greynoise.util import (
+    load_config,
+    validate_ip,
+    validate_similar_min_score,
+    validate_timeline_days,
+    validate_timeline_field_value,
+    validate_timeline_granularity,
+)
 
-if not structlog.is_configured():
-    configure_logging()
-LOGGER = structlog.get_logger()
+LOGGER = logging.getLogger(__name__)
 
 
 def initialize_cache(cache_max_size, cache_ttl):
@@ -39,18 +44,21 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
     """
 
     NAME = "GreyNoise"
-    API_VERSION = "v2"
-    EP_GNQL = "experimental/gnql"
-    EP_GNQL_STATS = "experimental/gnql/stats"
-    EP_INTERESTING = "interesting/{ip_address}"
-    EP_NOISE_MULTI = "noise/multi/quick"
-    EP_NOISE_CONTEXT = "noise/context/{ip_address}"
-    EP_NOISE_CONTEXT_MULTI = "noise/multi/context"
+    EP_GNQL = "v2/experimental/gnql"
+    EP_GNQL_STATS = "v2/experimental/gnql/stats"
+    EP_INTERESTING = "v2/interesting/{ip_address}"
+    EP_NOISE_MULTI = "v2/noise/multi/quick"
+    EP_NOISE_CONTEXT = "v2/noise/context/{ip_address}"
+    EP_NOISE_CONTEXT_MULTI = "v2/noise/multi/context"
     EP_COMMUNITY_IP = "v3/community/{ip_address}"
-    EP_META_METADATA = "meta/metadata"
+    EP_SIMILARITY_IP = "v3/similarity/ips/{ip_address}"
+    EP_TIMELINE_IP = "v3/noise/ips/{ip_address}/timeline"
+    EP_TIMELINE_HOURLY_IP = "v3/noise/ips/{ip_address}/hourly-summary"
+    EP_TIMELINE_DAILY_IP = "v3/noise/ips/{ip_address}/daily-summary"
+    EP_META_METADATA = "v2/meta/metadata"
     EP_PING = "ping"
-    EP_RIOT = "riot/{ip_address}"
-    EP_NOT_IMPLEMENTED = "request/{subcommand}"
+    EP_RIOT = "v2/riot/{ip_address}"
+    EP_NOT_IMPLEMENTED = "v2/request/{subcommand}"
     UNKNOWN_CODE_MESSAGE = "Code message unknown: {}"
     CODE_MESSAGES = {
         "0x00": "IP has never been observed scanning the Internet",
@@ -163,22 +171,16 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
             "User-Agent": " ".join(user_agent_parts),
             "key": self.api_key,
         }
-        if self.offering.lower() == "community":
-            url = "/".join([self.api_server, endpoint])
-        elif endpoint == self.EP_PING:
-            url = "/".join([self.api_server, endpoint])
-        else:
-            url = "/".join([self.api_server, self.API_VERSION, endpoint])
 
-        LOGGER.debug(
-            "Sending API request...",
-            url=url,
-            method=method,
-            headers=headers,
-            params=params,
-            json=json,
-            proxy=self.proxy,
-        )
+        url = "/".join([self.api_server, endpoint])
+
+        LOGGER.debug("Sending API request...URL: %s", url)
+        LOGGER.debug("Sending API request...method: %s", method)
+        LOGGER.debug("Sending API request...headers: %s", headers)
+        LOGGER.debug("Sending API request...params: %s", params)
+        LOGGER.debug("Sending API request...json: %s", json)
+        LOGGER.debug("Sending API request...proxy: %s", self.proxy)
+
         request_method = getattr(self.session, method)
         if self.proxy:
             proxies = {protocol: self.proxy for protocol in ("http", "https")}
@@ -200,12 +202,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
         else:
             body = response.text
 
-        LOGGER.debug(
-            "API response received",
-            status_code=response.status_code,
-            content_type=content_type,
-            body=body,
-        )
+        LOGGER.debug("API response received %s %s", response.status_code, body)
 
         if response.status_code == 429:
             raise RateLimitError()
@@ -268,9 +265,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                 "message": "Interesting report not supported with Community offering"
             }
         else:
-            LOGGER.debug(
-                "Reporting interesting IP: %s...", ip_address, ip_address=ip_address
-            )
+            LOGGER.debug("Reporting interesting IP: %s...", ip_address)
             validate_ip(ip_address)
 
             endpoint = self.EP_INTERESTING.format(ip_address=ip_address)
@@ -287,7 +282,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
         :rtype: dict
 
         """
-        LOGGER.debug("Getting context for %s...", ip_address, ip_address=ip_address)
+        LOGGER.debug("Getting context for %s...", ip_address)
         validate_ip(ip_address)
 
         if self.offering.lower() == "community":
@@ -320,18 +315,12 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
         response = self._request(endpoint)
         return response
 
-    def query(self, query, size=None, scroll=None):
+    def query(self, query, size=None, scroll=None, exclude_raw=False):
         """Run GNQL query."""
         if self.offering == "community":
             response = {"message": "GNQL not supported with Community offering"}
         else:
-            LOGGER.debug(
-                "Running GNQL query: %s...",
-                query,
-                query=query,
-                size=size,
-                scroll=scroll,
-            )
+            LOGGER.debug("Running GNQL query: %s %s %s...", query, size, scroll)
             params = {"query": query}
             if size is not None:
                 params["size"] = size
@@ -339,6 +328,11 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                 params["scroll"] = scroll
             response = self._request(self.EP_GNQL, params=params)
 
+        if exclude_raw:
+            if "data" in response:
+                for ip_data in response["data"]:
+                    ip_data.pop("raw_data")
+
         return response
 
     def quick(self, ip_addresses, include_invalid=False):  # pylint: disable=R0912,R0914
@@ -361,7 +355,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
             if isinstance(ip_addresses, str):
                 ip_addresses = ip_addresses.split(",")
 
-            LOGGER.debug("Getting noise status...", ip_addresses=ip_addresses)
+            LOGGER.debug("Getting noise status for %s...", ip_addresses)
 
             valid_ip_addresses = [
                 ip_address
@@ -388,7 +382,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                     )
                     for chunk in chunks:
                         api_result = self._request(
-                            self.EP_NOISE_MULTI, json={"ips": chunk}
+                            self.EP_NOISE_MULTI, method="post", json={"ips": chunk}
                         )
                         if isinstance(api_result, list):
                             api_results.extend(api_result)
@@ -408,7 +402,9 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                     valid_ip_addresses, self.IP_QUICK_CHECK_CHUNK_SIZE
                 )
                 for chunk in chunks:
-                    result = self._request(self.EP_NOISE_MULTI, json={"ips": chunk})
+                    result = self._request(
+                        self.EP_NOISE_MULTI, method="post", json={"ips": chunk}
+                    )
                     if isinstance(result, list):
                         results.extend(result)
                     else:
@@ -455,7 +451,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
             if isinstance(ip_addresses, str):
                 ip_addresses = ip_addresses.split(",")
 
-            LOGGER.debug("Getting noise context...", ip_addresses=ip_addresses)
+            LOGGER.debug("Getting noise context for IPs: %s", ip_addresses)
 
             valid_ip_addresses = [
                 ip_address
@@ -494,10 +490,6 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                         else:
                             api_results.append(api_result)
 
-                        for ip_address in valid_ip_addresses:
-                            if ip_address not in api_results:
-                                api_results.append({"ip": ip_address, "seen": False})
-
                     for result in api_results:
                         ip_address = result["ip"]
 
@@ -514,8 +506,9 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                 )
                 for chunk in chunks:
                     result = self._request(
-                        self.EP_NOISE_CONTEXT_MULTI, json={"ips": chunk}
+                        self.EP_NOISE_CONTEXT_MULTI, method="post", json={"ips": chunk}
                     )
+                    result = result["data"]
                     if isinstance(result, list):
                         results.extend(result)
                     else:
@@ -524,7 +517,12 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
             if include_invalid:
                 for ip_address in ip_addresses:
                     if ip_address not in valid_ip_addresses:
-                        results.append({"ip": ip_address, "seen": False})
+                        results.append(
+                            {
+                                "ip": ip_address,
+                                "noise": False,
+                            }
+                        )
 
         return results
 
@@ -533,7 +531,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
         if self.offering == "community":
             response = {"message": "Stats Query not supported with Community offering"}
         else:
-            LOGGER.debug("Running GNQL stats query: %s...", query, query=query)
+            LOGGER.debug("Running GNQL stats query: %s...", query)
             params = {"query": query}
             if count is not None:
                 params["count"] = count
@@ -571,7 +569,7 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
         if self.offering == "community":
             response = {"message": "RIOT lookup not supported with Community offering"}
         else:
-            LOGGER.debug("Checking RIOT for %s...", ip_address, ip_address=ip_address)
+            LOGGER.debug("Checking RIOT for %s...", ip_address)
             validate_ip(ip_address)
 
             endpoint = self.EP_RIOT.format(ip_address=ip_address)
@@ -581,3 +579,170 @@ class GreyNoise(object):  # pylint: disable=R0205,R0902
                 response["ip"] = ip_address
 
         return response
+
+    def similar(self, ip_address, limit=None, min_score=None):
+        """Query IP on the IP Similarity API
+
+        :param ip_address: IP address to use in the look-up.
+        :type ip_address: str
+        :param limit: Limit the number of matches returned by the endpoint
+        :type limit: str
+        :param limit: Limit the number of matches returned by the endpoint
+        :type limit: str
+        :return: Context for the IP address.
+        :rtype: dict
+
+
+        """
+        if self.offering == "community":
+            response = {
+                "message": "Similarity lookup not supported with Community offering"
+            }
+        else:
+            LOGGER.debug("Checking IP Sim results for %s...", ip_address)
+            validate_ip(ip_address)
+
+            if limit is None:
+                limit = 50
+
+            endpoint = self.EP_SIMILARITY_IP.format(ip_address=ip_address)
+            endpoint = endpoint + f"?limit={limit}"
+
+            if min_score:
+                validate_similar_min_score(min_score)
+                if min_score != 0:
+                    min_score = min_score / 100
+                endpoint = endpoint + f"&minimum_score={min_score}"
+
+            response = self._request(endpoint)
+
+            if "ip" not in response:
+                response["ip"] = ip_address
+
+        return response
+
+    def timeline(self, ip_address, field="classification", days=None, granularity=None):
+        """Query IP on the IP TimeSeries API
+
+        :param ip_address: IP address to use in the look-up.
+        :type ip_address: str
+        :param field: field name to use to retrieve timeline information
+        :type field: str
+        :param days: Number of days to show data for
+        :type days: int
+        :param granularity: Granularity of activity date ranges
+        :type granularity: str
+        :return: Context for the IP address.
+        :rtype: dict
+
+
+        """
+        if self.offering == "community":
+            response = {
+                "message": "Timeline lookup not supported with Community offering"
+            }
+        else:
+            LOGGER.debug("Checking IP Timeline results for %s...", ip_address)
+            validate_ip(ip_address)
+            if not field:
+                field = "classification"
+            validate_timeline_field_value(field)
+            if days:
+                validate_timeline_days(days)
+            if granularity:
+                validate_timeline_granularity(granularity)
+
+            endpoint = self.EP_TIMELINE_IP.format(ip_address=ip_address)
+            endpoint = endpoint + f"?field={field.lower()}"
+            if days:
+                endpoint = endpoint + f"&days={days}"
+            if granularity:
+                endpoint = endpoint + f"&granularity={granularity}"
+            response = self._request(endpoint)
+
+            if "ip" not in response:
+                response["ip"] = ip_address
+
+        return response
+
+    def timelinehourly(self, ip_address, days=None, cursor=None, limit=100):
+        """Query IP on the IP TimeSeries API
+
+        :param ip_address: IP address to use in the look-up.
+        :type ip_address: str
+        :param cursor:
+            The cursor is a pointer from which to start returning
+            results up to the limit
+        :type cursor: str
+        :param days: Number of days to show data for
+        :type days: int
+        :param limit: The total number of events to return in the response
+        :type limit: str
+        :return: Context for the IP address.
+        :rtype: dict
+
+
+        """
+        if self.offering == "community":
+            response = {
+                "message": "Timeline lookup not supported with Community offering"
+            }
+        else:
+            LOGGER.debug("Checking IP Timeline results for %s...", ip_address)
+            validate_ip(ip_address)
+            if days:
+                validate_timeline_days(days)
+
+            endpoint = self.EP_TIMELINE_HOURLY_IP.format(ip_address=ip_address)
+            endpoint = endpoint + f"?limit={limit}"
+            if days:
+                endpoint = endpoint + f"&days={days}"
+            if cursor:
+                endpoint = endpoint + f"&cursor={cursor}"
+            response = self._request(endpoint)
+
+            if "ip" not in response:
+                response["ip"] = ip_address
+
+        return response
+
+    def timelinedaily(self, ip_address, days=None, cursor=None, limit=50):
+        """Query IP on the IP TimeSeries API
+
+        :param ip_address: IP address to use in the look-up.
+        :type ip_address: str
+        :param cursor:
+            The cursor is a pointer from which to start returning
+            results up to the limit
+        :type cursor: str
+        :param days: Number of days to show data for
+        :type days: int
+        :param limit: The total number of events to return in the response
+        :type limit: str
+        :return: Context for the IP address.
+        :rtype: dict
+
+
+        """
+        if self.offering == "community":
+            response = {
+                "message": "Timeline lookup not supported with Community offering"
+            }
+        else:
+            LOGGER.debug("Checking IP Timeline results for %s...", ip_address)
+            validate_ip(ip_address)
+            if days:
+                validate_timeline_days(days)
+
+            endpoint = self.EP_TIMELINE_DAILY_IP.format(ip_address=ip_address)
+            endpoint = endpoint + f"?limit={limit}"
+            if days:
+                endpoint = endpoint + f"&days={days}"
+            if cursor:
+                endpoint = endpoint + f"&cursor={cursor}"
+            response = self._request(endpoint)
+
+            if "ip" not in response:
+                response["ip"] = ip_address
+
+        return response
