@@ -4,9 +4,9 @@ Decorators used to add common functionality to subcommands.
 
 """
 import functools
+import logging
 
 import click
-import structlog
 from requests.exceptions import RequestException
 
 from greynoise.api import GreyNoise
@@ -15,7 +15,7 @@ from greynoise.cli.parameter import ip_addresses_parameter
 from greynoise.exceptions import RequestFailure
 from greynoise.util import load_config
 
-LOGGER = structlog.get_logger()
+LOGGER = logging.getLogger(__name__)
 
 
 def echo_result(function):
@@ -70,12 +70,14 @@ def handle_exceptions(function):
             else:
                 error_message = "API error: {}".format(body)
             LOGGER.error(error_message)
-            click.echo(error_message)
             click.get_current_context().exit(-1)
         except RequestException as exception:
             error_message = "API error: {}".format(exception)
             LOGGER.error(error_message)
-            click.echo(error_message)
+            click.get_current_context().exit(-1)
+        except ValueError as exception:
+            error_message = "Validator error: {}".format(exception)
+            LOGGER.error(error_message)
             click.get_current_context().exit(-1)
 
     return wrapper
