@@ -1,14 +1,12 @@
 """Utility functions."""
-import ipaddress
 import logging
 import os
-import sys
+from ipaddress import IPv6Address, ip_address
 
-import structlog
 from six.moves.configparser import ConfigParser
 
 CONFIG_FILE = os.path.expanduser(os.path.join("~", ".config", "greynoise", "config"))
-LOGGER = structlog.get_logger()
+LOGGER = logging.getLogger(__name__)
 
 DEFAULT_CONFIG = {
     "api_key": "",
@@ -19,27 +17,6 @@ DEFAULT_CONFIG = {
 }
 
 
-def configure_logging():
-    """Configure logging."""
-    logging.basicConfig(stream=sys.stderr, format="%(message)s", level=logging.CRITICAL)
-    logging.getLogger("greynoise").setLevel(logging.WARNING)
-    structlog.configure(
-        processors=[
-            structlog.stdlib.add_logger_name,
-            structlog.stdlib.add_log_level,
-            structlog.stdlib.PositionalArgumentsFormatter(),
-            structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M.%S"),
-            structlog.processors.StackInfoRenderer(),
-            structlog.processors.format_exc_info,
-            structlog.dev.ConsoleRenderer(),
-        ],
-        context_class=dict,
-        logger_factory=structlog.stdlib.LoggerFactory(),
-        wrapper_class=structlog.stdlib.BoundLogger,
-        cache_logger_on_first_use=True,
-    )
-
-
 def load_config():
     """Load configuration.
 
@@ -54,27 +31,21 @@ def load_config():
     config_parser.add_section("greynoise")
 
     if os.path.isfile(CONFIG_FILE):
-        LOGGER.debug("Parsing configuration file: %s...", CONFIG_FILE, path=CONFIG_FILE)
+        LOGGER.debug("Parsing configuration file: %s...", CONFIG_FILE)
         with open(CONFIG_FILE) as config_file:
             config_parser.read_file(config_file)
     else:
-        LOGGER.debug("Configuration file not found: %s", CONFIG_FILE, path=CONFIG_FILE)
+        LOGGER.debug("Configuration file not found: %s", CONFIG_FILE)
 
     if "GREYNOISE_API_KEY" in os.environ:
         api_key = os.environ["GREYNOISE_API_KEY"]
-        LOGGER.debug(
-            "API key found in environment variable: %s", api_key, api_key=api_key
-        )
+        LOGGER.debug("API key found in environment variable: %s", api_key)
         # Environment variable takes precedence over configuration file content
         config_parser.set("greynoise", "api_key", api_key)
 
     if "GREYNOISE_API_SERVER" in os.environ:
         api_server = os.environ["GREYNOISE_API_SERVER"]
-        LOGGER.debug(
-            "API server found in environment variable: %s",
-            api_server,
-            api_server=api_server,
-        )
+        LOGGER.debug("API server found in environment variable: %s", api_server)
         # Environment variable takes precedence over configuration file content
         config_parser.set("greynoise", "api_server", api_server)
 
@@ -87,32 +58,21 @@ def load_config():
                 "GREYNOISE_TIMEOUT environment variable "
                 "cannot be converted to an integer: %r",
                 timeout,
-                timeout=timeout,
             )
         else:
-            LOGGER.debug(
-                "Timeout found in environment variable: %s", timeout, timeout=timeout
-            )
+            LOGGER.debug("Timeout found in environment variable: %s", timeout)
             # Environment variable takes precedence over configuration file content
             config_parser.set("greynoise", "timeout", timeout)
 
     if "GREYNOISE_PROXY" in os.environ:
         proxy = os.environ["GREYNOISE_PROXY"]
-        LOGGER.debug(
-            "Proxy found in environment variable: %s",
-            proxy,
-            proxy=proxy,
-        )
+        LOGGER.debug("Proxy found in environment variable: %s", proxy)
         # Environment variable takes precedence over configuration file content
         config_parser.set("greynoise", "proxy", proxy)
 
     if "GREYNOISE_OFFERING" in os.environ:
         offering = os.environ["GREYNOISE_OFFERING"]
-        LOGGER.debug(
-            "Offering found in environment variable: %s",
-            offering,
-            offering=offering,
-        )
+        LOGGER.debug("Offering found in environment variable: %s", offering)
         # Environment variable takes precedence over configuration file content
         config_parser.set("greynoise", "offering", offering)
 
@@ -148,7 +108,7 @@ def save_config(config):
         config_parser.write(config_file)
 
 
-def validate_ip(ip_address, strict=True, print_warning=True):
+def validate_ip(ip, strict=True, print_warning=True):
     """Check if the IPv4 address is valid.
 
     :param ip_address: IPv4 address value to validate.
@@ -161,26 +121,110 @@ def validate_ip(ip_address, strict=True, print_warning=True):
 
     """
     is_valid = False
+    error_message = ""
 
     try:
-        ipaddress.ip_address(ip_address)
+        ip_address(ip)
         is_valid = True
     except ValueError:
         if print_warning:
-            error_message = "Invalid IP address: {!r}".format(ip_address)
-            LOGGER.warning(error_message, ip_address=ip_address)
+            error_message = "Invalid IP address: {!r}".format(ip)
+            LOGGER.warning(error_message)
         if strict:
             raise ValueError(error_message)
         return False
 
     if is_valid:
-        is_routable = ipaddress.ip_address(ip_address).is_global
-        if is_routable:
-            return True
-        else:
+        if type(ip_address(ip)) is IPv6Address:
+            error_message = "IPv6 addresses are not supported: {!r}".format(ip)
             if print_warning:
-                error_message = "Non-Routable IP address: {!r}".format(ip_address)
-                LOGGER.warning(error_message, ip_address=ip_address)
+                LOGGER.warning(error_message)
             if strict:
                 raise ValueError(error_message)
             return False
+        else:
+            is_routable = ip_address(ip).is_global
+            if is_routable:
+                return True
+            else:
+                error_message = "Non-Routable IP address: {!r}".format(ip)
+                if print_warning:
+                    LOGGER.warning(error_message)
+                if strict:
+                    raise ValueError(error_message)
+                return False
+
+
+def validate_timeline_field_value(field):
+    """Check if the Timeline Field value is valid.
+
+    :param field: field value to validate.
+    :type field: str
+
+    """
+    valid_field_names = [
+        "destination_port",
+        "http_path",
+        "http_user_agent",
+        "source_asn",
+        "source_org",
+        "source_rdns",
+        "tag_ids",
+        "classification",
+    ]
+
+    if field in valid_field_names:
+        return True
+    else:
+        raise ValueError(
+            f"Field must be one of the following values: {valid_field_names}"
+        )
+
+
+def validate_timeline_days(days):
+    """Check if the Timeline Days value is valid.
+
+    :param days: field value to validate.
+    :type days: str
+
+    """
+    if isinstance(days, str):
+        raise ValueError(
+            "Days must be a valid integer between 1 and 30.  Current input is a "
+            "string."
+        )
+    if isinstance(days, int) and 1 <= int(days) <= 30:
+        return True
+    else:
+        raise ValueError("Days must be a valid integer between 1 and 30.")
+
+
+def validate_timeline_granularity(granularity):
+    """Check if the Timeline granularity value is valid.
+
+    :param granularity: field value to validate.
+    :type granularity: str
+
+    """
+    if granularity != "1h" and granularity != "1d":
+        raise ValueError("Granularity currently only supports a value of 1d or 1h")
+    else:
+        return True
+
+
+def validate_similar_min_score(min_score):
+    """Check if the Similarity min_score value is valid.
+
+    :param min_score: field value to validate.
+    :type min_score: str
+
+    """
+    if isinstance(min_score, str):
+        raise ValueError(
+            "Min Score must be a valid integer between 0 and 100.  Current input is a "
+            "string."
+        )
+    if isinstance(min_score, int) and 0 <= int(min_score) <= 100:
+        return True
+    else:
+        raise ValueError("Min Score must be a valid integer between 0 and 100.")
